/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

Application
    rhoTurbFoam+laplacian

Description
    Transient solver for compressible, turbulent flow + heat conduction in solids.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "basicThermo.H"
#include "compressible/turbulenceModel/turbulenceModel.H"
#include "fixedGradientFvPatchFields.H"
#include "typeInfo.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMeshes.H"
#   include "createFields.H"
#   include "readPISOControls.H"
#   include "initContinuityErrs.H"
#   include "readTimeControls.H"
#   include "setInitialDeltaT.H"


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    double deltagrad=0.2;
    int corrT=0;

    while (runTime.run())
    {        

#       include "readTimeControls.H"
#       include "CourantNo.H"
#       include "setDeltaT.H"

        runTime++;
	deltagrad=0.2;
	corrT=0;

        Info<< "\n\nTime = " << runTime.timeName() << endl;

	//if the flux difference at the boundary interface (deltagrad)
	//is higher than 0.0001 recompute
	//note: the loop is done at least one time for each time step and
	//maximum nCorr times

	while ( mag(deltagrad)>0.0001  && corrT < nCorr)
	  {

	    // number of pseudo time steps
	    corrT=corrT+1;	    
	
	    // solve the NS equations in the fluid

#       include "rhoEqn.H"
#       include "UEqn.H"

	    // --- PISO loop
	    for (int corr=1; corr<=nCorr; corr++)
	      {
#           include "hEqn.H"
#           include "pEqn.H"
	      }

	    turbulence->correct();


	// get the label for the interface boundary called solid_interface in solid region
 	label solidInterface = mesh2.boundaryMesh().findPatchID("solid_interface");
	// get the label for the interface boundary called fluid_interface in fluid region
 	label fluidInterface = mesh1.boundaryMesh().findPatchID("fluid_interface");
	
	if (-1 == solidInterface || -1 == fluidInterface)
	{
		error err("no solid or fluid interface");
		err.exit();
	}
	
	
	// reads the scalar Temperature field at the first internal boundary
	scalarField TsolidInterface = T2.boundaryField()[solidInterface];
 	volScalarField& T1 = thermo->T();
 	scalarField TfluidInterface = T1.boundaryField()[fluidInterface];
	
	// calculates the gradient of Temperature in all the field
 	volVectorField gradT1cal = fvc::grad(T1);
 	volVectorField gradT2cal = fvc::grad(T2);

	// take the reference to the gradient value imposed to solid region
 	fixedGradientFvPatchScalarField& gradTsolidInterface = 
	refCast<fixedGradientFvPatchScalarField>(T2.boundaryField()[solidInterface]);
	scalarField& gradT2 =  gradTsolidInterface.gradient();
	
	//calculate the vector field of unit surface normals
	surfaceVectorField unitNormals1 = mesh1.Sf()/mesh1.magSf();
	surfaceVectorField unitNormals2 = mesh2.Sf()/mesh2.magSf();
	
	//calculate the projection of the gradient onto the normal of the boundary
	scalarField gradT1norm = gradT1cal.boundaryField()[fluidInterface] & unitNormals1.boundaryField()[fluidInterface];
	scalarField gradT2norm = gradT2cal.boundaryField()[solidInterface] & unitNormals2.boundaryField()[solidInterface];

	// set the gradient imposed to solid region equal to the gradient extracted
	// from fluid region multiplied by the ratio of the conductivy of the
	// two zones
	scalarField coeff = thermo->alpha()*thermo->rho()*thermo->Cp()/K2;
	gradT2 = -gradT1norm*coeff;

	deltagrad = max(gradT2norm)-max(-gradT1norm*coeff);

	Info << "grad T1 " << max(gradT1norm) <<"\n";

	Info << "delta grad " << corrT << " --  " << max(gradT2norm)-max(-gradT1norm*coeff) <<"\n";



        // solve temperature field in the solid

        for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
        {
            solve
            (
                fvm::ddt(T2) - fvm::laplacian(DT2, T2)
            );
        }


	//Info << " delta T " << corrT << " --  " << max(T1.boundaryField()[fluidInterface])  - max(TsolidInterface) <<"\n";

        Info<< "\n\nTime = " << runTime.timeName() << endl;

	Info << " T solid interface " << max( TsolidInterface ) <<"\n";


	// set the Temperature imposed to fluid region equal to temperature of 
	// the boundary in solid region

 	T1.boundaryField()[fluidInterface] ==  TsolidInterface;

	Info << " T fluid interface " << max(T1.boundaryField()[fluidInterface]) <<"\n";

        runTime.write();

	  }

        Info<< "\nExecutionTime = "
             << runTime.elapsedCpuTime()
             << " s\n" << endl;
 
    }

    Info<< "\n end\n";

    return(0);
}


// ************************************************************************* //
